hglmm <- function(formula,data,cells=NULL,K=NULL,par0=NULL,method="lm",
                  lmc=10,expo=TRUE,mixture=FALSE,tolerance=NULL,
                  overpar=FALSE,digits=NULL,verbose=FALSE,itmax=200,
                  data.name=NULL,contr=c("treatment","sum","helmert"),
                  crit=NULL,breaks=NULL,
                  use.anal=FALSE,ca=FALSE) {

#
# Function hglmm.  To carry out the fitting of a Hidden Markov
# model structure given Poisson data the parameter of which depends
# in part upon the unobserved state of an associated Markov chain.
#

# Initialize:
if(verbose) {
	cat('\n')
	cat('Initializing ...\n\n')
}

# Set the tolerance if this is NULL:
if(is.null(tolerance)) tolerance <- sqrt(.Machine$double.eps)

# Issue warning if expo=FALSE and method is not "em".
if(!(expo | method == "em"))
   warning("Setting expo=FALSE is ignored unless method==\"em\".\n")

# Set the contrasts:
contr <- match.arg(contr)
contr <- c(paste("contr",contr,sep="."),"contr.poly")
OC    <- options(contrasts=contr)
on.exit(options(OC))

# Put together a data name tag for the output.
if(is.null(data.name)) data.name <- deparse(substitute(data))

# Build the ``cells'' factor.
if(is.null(cells)) {
    data[,"cf"] <- factor(rep(1,nrow(data)))
} else {
    data[,"cf"] <- interaction(data[,cells])
}

# Put state into the formula.
cform    <- as.character(formula)
if(cform[3]=="1") {
    cform[3] <- "0+state"
} else {
    cform[3] <- paste("0+state+",cform[3])
}
fmla     <- as.formula(paste(cform[2],cform[3],sep=" ~ "))

# Initialize the parameters.
if(is.null(par0)) {
	if(is.null(K)) stop('One of K or par0 must be specified.\n')
} else K <- nrow(par0$tpm)

par.all  <- init.all(fmla,data,K,par0,mixture,breaks,expo=expo)
phi      <- par.all$phi
if(expo) {
	rho <- par.all$rho
	theta <- c(rho,phi)
} else {
	tpm    <- par.all$tpm
	tpm.nr <- c(t(tpm[,-K]))
	theta  <- c(tpm.nr,phi)
}

# Build the replicated data frame.
M     <- nrow(data)
rd.df <- as.data.frame(lapply(data,function(x,K,M){rep(x,rep(K,M))},K=K,M=M))
rd.df$state <- factor(rep(1:K,M))

if(method=="lm") {
	if(is.null(crit)) crit <- "ABSGRD"
	rslt <- hglmm.lm(formula=fmla,data=rd.df,K=K,theta=theta,lmc=lmc,
                             mixture=mixture,itmax=itmax,crit=crit,
                             tolerance=tolerance,overpar=overpar,digits=digits,
                             verbose=verbose)
} else if(method=="sw") {
	if(is.null(crit)) crit <- "ABSGRD"
	start <- hglmm.em(formula=fmla,data=rd.df,K=K,theta=theta,
                         itmax=itmax,crit="CLL",mixture=mixture,
                         tolerance=1e-3,overpar=FALSE,digits=digits,
                         verbose=verbose)$theta
	rslt <- hglmm.lm(formula=fmla,data=rd.df,K=K,theta=start,lmc=lmc,
                             mixture=mixture,itmax=itmax,crit=crit,
                             tolerance=tolerance,overpar=overpar,digits=digits,
                             verbose=verbose)
} else if(method=="em") {
	if(is.null(crit)) crit <- "CLL"
	rslt <- hglmm.em(formula=fmla,data=rd.df,K=K,theta=theta,expo=expo,
                         itmax=itmax,crit=crit,mixture=mixture,
                         tolerance=tolerance,overpar=overpar,digits=digits,
                         verbose=verbose)
} else if(method=="bf") {
	rslt <- hglmm.bf(formula=fmla,data=rd.df,K=K,theta=theta,
                         mixture=mixture,itmax=200,tolerance=tolerance,
                         overpar=overpar,digits=digits,verbose=verbose)
} else if (method == "nl") {
        rslt <- hglmm.nl(formula=fmla, data=rd.df, K=K, theta=theta,
                mixture = mixture, itmax = itmax, tolerance = tolerance,
                overpar = overpar, use.anal=use.anal,ca=ca, verbose = verbose)
} else
	stop(paste("Unrecognized method ",method,".\n",sep=""))

rslt$data.name <- data.name
rslt$method    <- method
rslt$tolerance <- tolerance
rslt$cells     <- cells
class(rslt)    <- "hglmm"
rslt
}
