hglmm.em <- function(formula,data,K,theta,expo=TRUE,mixture=FALSE,
                     itmax=200,crit="CLL",tolerance=NULL,overpar=FALSE,
		     digits=NULL,verbose=FALSE) {
#
# Function hglmm.em.  The engine to carry out the fitting of a
# Hidden Markov model using the EM algorithm.
#

# Pick out the index of the stopping criterion:
if(crit=="ABSGRD")
	stop(paste("Stopping criterion \"ABSGRD\" cannot be",
                   "used with EM algorithm.\n"))
icrit <- match(crit,c('CLL','L2','Linf'))
if(is.na(icrit)) stop(paste("Stopping criterion",crit,"not recognized.\n"))

# Do some initial housekeeping.
fmla <- formula
np   <- K*(K-1)
if(expo) {
	rho <- theta[1:np]
	tpm <- fix.tpm(rho,K,TRUE)
} else {
	tpm.nr <- theta[1:np]
	tpm <- fix.tpm(tpm.nr,K,FALSE)
}
phi    <- theta[-(1:np)]
old.ll <- -Inf
oath   <- theta
if(is.null(digits)) digits <- 2+ceiling(abs(log10(tolerance)))
scrit  <- numeric(4)
if(is.null(tolerance)) tolerance <- sqrt(.Machine$double.eps)

# Build the first go at lambda and fy:
X   <- model.matrix(fmla[-2],data=data)
lam <- exp(X%*%phi)
y   <- data[,as.character(fmla)[2]]
fy  <- dpois(y,lam)
fy[is.na(fy)] <- 1
par.mod       <- list(phi=phi,phi.over=NULL,lambda=lam,fy=fy)

# Set the level below which probabilities are considered to
# be noise:
epsilon <- .Machine$double.eps

if(verbose) cat('      Initialization complete ...\n\n')

# Update:
em.step <- 1
if(verbose) cat('Repeating ...\n\n')

repeat{ # Swear again; i.e. recurse.
	if(verbose) cat(paste('EM step ',em.step,':\n',sep=''))

# Calculate the recursive probabilities.
	fy <- par.mod$fy
	rp <- recurse(fy,tpm,data$cf,epsilon,mixture)

# Calculate the parameters.
	tpm <- rp$tpm
	if(expo) rho <- fix.rho(tpm) else tpm.nr <- c(t(tpm[,1:(K-1)]))
	data$weights <- c(rp$gamma)
	par.mod <- revise.model(fmla,data,overpar=overpar)
	phi <- par.mod$phi

# Test for convergence:
	theta <- if(expo) c(rho,phi) else c(tpm.nr,phi)
	ll <-  sum(log(abs(rp$llc)))
	scrit[1] <- if(old.ll > -Inf)
                        (ll - old.ll)/(abs(ll) + tolerance)
                    else Inf
        scrit[2] <- sqrt(sum((oath-theta)^2))/(sqrt(sum(theta^2)) + tolerance)
        scrit[3] <- max(abs(oath-theta))/(max(abs(theta)) + tolerance)
        if(verbose){
                cat('     Log-likelihood: ',
                format(round(ll,digits)),'\n',sep='')
                cat('     Scaled decrease in log-likelihood: ',
                    format(round(scrit[1],digits)),'\n',sep='')
                cat('     Scaled root-SS of change in coef.: ',
                    format(round(scrit[2],digits)),'\n',sep='')
                cat('     Scaled max. abs. change in coef.: ',
                    format(round(scrit[3],digits)),'\n',sep='')
        }

        if(scrit[icrit] < tolerance) {
                        converged <- TRUE
                        nstep     <- em.step
                        break
        }

	if(em.step >= itmax) {
		cat('Failed to converge in ',itmax,' EM steps.\n',sep='')
		converged <- FALSE
		nstep     <- em.step
		break
	}
        oath    <- theta
        old.ll  <- ll
	em.step <- em.step + 1
}

M    <- nrow(data)/K
c1   <- (1:M)[apply(matrix(fy,nrow=K),2,sum)<(K*epsilon)]
c2   <- (1:M)[rp$llc<0]
clip <- list(c1=c1,c2=c2)

ispd         <- revise.ispd(tpm)
params       <- list(tpm=tpm,ispd=ispd,phi=phi,phi.over=par.mod$phi.over)
miss.frac    <- sum(is.na(data$y))/nrow(data)
lam          <- par.mod$lambda
fy           <- par.mod$fy
data$weights <- NULL

names(theta)[1:(K*(K-1))] <- if(expo)
      paste('rho_',t(row(tpm)[,-K]),t(col(tpm)[,-K]),sep='')
else
      paste('p_',t(row(tpm)[,-K]),t(col(tpm)[,-K]),sep='')

list(params=params,theta=theta,miss.frac=miss.frac,
     mixture=mixture,log.like=ll,clip=clip,expo=expo,
     converged=converged,nstep=nstep,
     formula=fmla,contrasts=options("contrasts"),lambda=lam,
     fy=fy,data=data)

}
