hglmm.lm <- function(formula,data,K,theta,lmc=10,mixture=FALSE,
                         itmax=200,crit,tolerance=NULL,
                         overpar=FALSE,digits=NULL,verbose=FALSE) {
#
# Function hglmm.lm.  The engine to carry out the fitting of a
# Hidden Markov model using the Levenberg-Marquardt algorithm.
#

if(mixture)
	stop(paste("Sorry; can't do mixtures by the",
                   "Levenberg-Marquardt algorithm yet.\n"))

# Pick out the index of the stopping criterion:
icrit <- match(crit, c("CLL", "L2", "Linf","ABSGRD"))
if(is.na(icrit)) stop('Stopping criterion not recognized.\n')

# Do some initial housekeeping.
fmla   <- formula
npar   <- length(theta)
npro   <- K*(K-1)
X      <- model.matrix(fmla[-2],data)
y      <- data[,as.character(fmla)[2]]
cf     <- data$cf
old.ll <- -Inf
oath   <- theta
if(is.null(digits)) digits <- 2+ceiling(abs(log10(tolerance)))
if(is.null(tolerance)) tolerance <- sqrt(.Machine$double.eps)
scrit  <- numeric(4)

# Update:
if(verbose) cat('Repeating ...\n\n')
lm.step <- 1

repeat { # Swear again; i.e. recurse.
	if(verbose) cat(paste('Lev.-Marq. step ',lm.step,':\n',sep=''))
	xxx   <- lmstep(theta,K,X,y,cf,lmc)
	theta <- xxx$theta
	rho   <- theta[1:npro]
	tpm   <- fix.tpm(rho,K,TRUE)
	if(identical(all.equal(tpm,diag(K)),TRUE)) {
		return(list(converged=FALSE,message="tpm equals identity"))
	}
	ll    <- xxx$ll
	grad  <- xxx$grad
	scrit[1] <- if(old.ll > -Inf)
                        (ll - old.ll)/(abs(ll) + tolerance)
                    else Inf
        scrit[2] <- sqrt(sum((theta-oath)^2))/(sqrt(sum(theta^2)) + tolerance)
        scrit[3] <- max(abs(theta-oath))/(max(abs(theta)) + tolerance)
        scrit[4] <- sum(abs(grad))
        if(verbose){
                cat('     Log-likelihood: ',
                format(round(ll,digits)),'\n',sep='')
                cat('     Scaled decrease in log-likelihood: ',
                    format(round(scrit[1],digits)),'\n',sep='')
                cat('     Scaled root-SS of change in coef.: ',
                    format(round(scrit[2],digits)),'\n',sep='')
                cat('     Scaled max. abs. change in coef.: ',
                    format(round(scrit[3],digits)),'\n',sep='')
                cat('     Sum abs. val. of grad. vector.: ',
                    format(round(scrit[4],digits)),'\n',sep='')
        }

        if(scrit[icrit] < tolerance) {
                        converged <- TRUE
                        nstep <- lm.step
                        nstep <- lm.step
                        break
        }
        if(lm.step >= itmax) {
                cat(paste("Failed to converge in ",itmax,
                          " Levenberg-Marquardt steps.\n",sep=""))
                converged <- FALSE
                nstep     <- lm.step
                break
        }
        oath    <- theta
        old.ll  <- ll
        lm.step <- lm.step + 1
        lmc     <- 0.1*xxx$lmc
}

xxx   <- newstep(theta,K,X,y,cf)
theta <- xxx$theta
phi   <- theta[-(1:npro)]
rho   <- theta[1:npro]
tpm   <- fix.tpm(rho,K,TRUE)
ispd  <- revise.ispd(tpm)
mf    <- sum(is.na(data$y))/nrow(data)
hess  <- xxx$hess
grad  <- xxx$grad
lam   <- exp(X%*%phi)
fy    <- dpois(y,lam)
fy[is.na(fy)] <- 1
nms   <- dimnames(X)[[2]]
nms[nms=="(Intercept)"] <- "mu"
names(phi) <- nms

if(overpar) {
# Do a bit of cosmetix:  Arrange to get the overparameterized
# version of the model.
	epsilon       <- .Machine$double.eps
	data$weights  <- c(recurse(fy,tpm,cf,epsilon,mixture=FALSE)$gamma)
	rrr    <- glm(fmla, data=data, weights=weights, family=poisson,
	              na.action=na.omit)
	phi.em <- coef(rrr)
	nms    <- names(phi.em)
	names(phi.em)[nms=='(Intercept)']   <- 'mu'
	names(rrr$coef)[nms=='(Intercept)'] <- 'mu'
	if(!identical(all.equal(phi,phi.em),TRUE)) {
		warning("EM step changed the model parameters.\n")
		phi.over.em <- dummy.coef(rrr)
		rrr$coef <- phi
	} else {
		phi.em <- NULL
		phi.over.em <- NULL
	}
	phi.over <- dummy.coef(rrr)
	nms      <- names(phi.over)
	names(phi.over)[nms=='(Intercept)']   <- 'mu'
} else {
	phi.over <- NULL
	phi.em   <- NULL
}

theta <- c(rho,phi)

names(theta)[1:(K*(K-1))] <-
      paste('rho_',t(row(tpm)[,-K]),t(col(tpm)[,-K]),sep='')
par <- list(tpm=tpm,ispd=ispd,phi=phi,phi.over=phi.over)
if(!is.null(phi.em)) {
	par$phi.em <- phi.em
	par$phi.over.em <- phi.over.em
}
data$weights <- NULL
rownames(hess) <- colnames(hess) <- names(theta)
names(grad) <- names(theta)

list(params=par,theta=theta,hessian=hess,grad=grad,log.like=ll,
     converged=converged,nstep=nstep,mixture=mixture,
     expo=TRUE,formula=fmla,contrasts=options("contrasts"),
     miss.frac=mf,lambda=lam,fy=fy,data=data)
}
