\name{hglmm}
\alias{hglmm}
\title{ Fit hidden generalised linear Markov models.}
\description{
  Fits a log linear Poisson model to a data set where the
  response in each \dQuote{cell} of the model consists of a
  time series of Poisson counts whose serial dependence is
  modelled by a hidden Markov model.
}
\usage{
  hglmm(formula, data, cells=NULL, K = NULL, par0 = NULL, method =
  "lm", lmc = 10, expo = TRUE, mixture = FALSE, tolerance = NULL,
  overpar=FALSE, digits = NULL, verbose = FALSE, itmax = 200,
  data.name = NULL, contr = c("treatment", "sum", "helmert"),
  crit = NULL, breaks = NULL, use.anal=FALSE, ca = FALSE)

}
\arguments{
  \item{formula}{ A model formula specifying the linear predictor for
  the log linear model. The formula should \emph{not} include
  \code{state} as a predictor variable.  The variable \code{state}
  gets added to the formula automatically.}
  \item{data}{ A data frame with columns specifying the response
  and the predictor varibles in the model.}
  \item{cells}{ A character vector giving the names of the factors
  (columns of the \code{data} data frame) which determine what the
  \dQuote{cells} of the model are considered to be.  The cells correspond
  to the combinations of levels of the factors named by \code{cells}.}
  \item{K}{The number of states of the hidden Markov model.
  If \code{K} is not specified and \code{par0} (see below) \emph{is}
  specified and has a component \code{tpm} then \code{K} is set
  equal to \code{nrow(tpm)}.  An error is given in this setting if
  \code{K} is specified to a value different from \code{nrow(tpm)}.}
  \item{par0}{ A list comprising initial parameter estimates.  It
  may have components \code{tpm} (an estimate of the transition
  probability matrix) and \code{phi} (a vector of estimates
  of the coefficients in the linear predictor in the log linear model).}
  \item{method}{ The method used to fit the model; it may be \dQuote{lm}
  (Levenberg-Marquardt algorithm, the default), \dQuote{sw} (start
  with the EM algorithm and then swap to Levenberg-Marquardt),
  \dQuote{em} (EM algorithm), \dQuote{bf} (\dQuote{brute force}; calls upon
  \code{\link{optim}()} to do the heavy lifting) or \dQuote{nl} (use
  function \code{\link{nlm}()}, non-linear minimization).}
  \item{lmc}{ The initial \dQuote{Levenberg-Marquardt constant}. }
  \item{expo}{ Logical variable; if \code{TRUE} then the transition
  probabilities of the chain are parameterized in an exponential
  form.  It is ignored (with a warning if it has been set to
  \code{FALSE}) unless the method used is \dQuote{em}.}
  \item{mixture}{ Logical scalar; if TRUE then a mixture model
  (rather than a general hidden Markov model) is fitted.  Currently
  mixture models can only be fitted using the EM algorithm
  (method = \dQuote{em}.}
  \item{tolerance}{ The convergence tolerance to be used. If not
  supplied defaults to \code{sqrt(.Machine$double.eps)}.}
  \item{overpar}{Logical scalar.  Should the overparameterized
  form of the model be returned?}
  \item{digits}{ The number of digits to which \dQuote{progress reports}
  are printed when \code{verbose} is \code{TRUE}.  Not used if method
  is \code{bf} or \code{nl}.}
  \item{verbose}{ Logical scalar; if \code{TRUE} progress reports
  are printed out at appropriate points in the iteration.}
  \item{itmax}{ The maximum number of iterative steps to take.  For
  methods \dQuote{em} and \dQuote{lm}, if
  convergence is not achieved by \code{itmax} steps, the function
  gives up, prints a message to this effect, and returns a value
  with a component \code{converged=FALSE}.  Has a slightly different
  meaning for methods \dQuote{bf} and \dQuote{nl}.}
  \item{data.name}{ A name to identify the data set to which the
  model was fitted.  If not supplied it is taken from the \code{data}
  argument using \code{deparse(substitute(data))}.}
  \item{contr}{ A text string specifying the contrasts (in respect of
  unordered factors) to use when producing the design matrix from
  the model formula.  Possible values "helmert", "sum", "treatment".
  Defaults to "treatment".  Partial matching is used, so abbreviation
  is permitted.}
  \item{crit}{ A text string specifying the stopping criterion
  to be used.  Possible values are \dQuote{CLL} (scaled change in log
  likelihood), \dQuote{L2} (scaled square root of sum of squares of the
  changes in the parameter estimates), \dQuote{Linf} (scaled maximum of
  the absolute value of the changes in the parameter estimates),
  and \dQuote{ABSGRD} (the sum of the absolute values of the entries
  of the gradient vector).  The latter only makes sense for the
  Levenberg-Marquardt algorithm.  The \code{bf} method can only
  use \dQuote{CLL}; the \code{nl} method seems to use a combination of
  \dQuote{ABSGRD} and \dQuote{CLL}.}
  \item{breaks}{ A vector of \code{K+1} values used to construct
  a set of guesses at the states corresponding to each observation.
  These are in turn used to calculate an initial estimate of the
  transition probability matrix.}
  \item{use.anal}{ Logical scalar; should \dQuote{analyticals} gradient
  and hessian be used?  This argument has effect only when the
  method is \dQuote{nl}.}
  \item{ca}{Logical scalar; \dQuote{check analyticals}; used when the
  method is \dQuote{nl}, and passed on to \code{\link{nlm}()}.}
}
\value{
  A list with components:
  \item{params}{Another list providing the estimates of the
  parameters of the model.  It has components \code{tpm} (the
  estimated transition probability matrix), \code{ispd} (the
  estimated initial state probability distribution), \code{phi}
  (the vector of estimated coefficients of the linear predictor in
  the log linear model), and \code{phi.over}. This last consists of
  the coefficients of the linear predictor in their redundant form
  corresponding to the overparameterized expression of the model
  if \code{overpar} is \code{TRUE}, otherwise it is \code{NULL}.}
  \item{theta}{The vector of parameter estimates that the estimation
  procedure actually works with, consisting of the catenation of
  the non-redundant parameterization of the transition probability
  matrix and the vector \code{phi}.}
  \item{hessian}{(Not present for the \dQuote{em} method.) The hessian
  matrix, i.e. the matrix of second partial derivatives of the
  log likelihood, evaluated at the final parameter estimates.
  The inverse of the negative of this matrix constitutes an estimate
  of the covariance matrix of the parameter estimates.}
  \item{grad}{(Not present for the \dQuote{em} method.) The gradient
  of the log likelihood at the final parameter estimates; it
  \emph{should} be effectively the zero vector.}
  \item{log.like}{The value of the log likelihood of the model evaluated
  at the parameter estimates, i.e. the (approximately) maximal value
  of the log likelihood.}
  \item{converged}{A logical scalar.  For the \dQuote{lm}, \dQuote{swp} and
  \dQuote{em} methods it is \code{TRUE} if convergence is achieved within
  \code{itmax} iterations and \code{FALSE} otherwise.  For the \dQuote{bf}
  method \code{converged} is \code{TRUE} if the \code{convergence}
  component of the object returned by \code{\link{optim}()}.
  is equal to 0 and \code{FALSE} otherwise.  For the \dQuote{nl} method
  \code{converged} is \code{TRUE} if the \code{code} component of the
  object returned by \code{\link{nlm}()} is less than or equal to 2.}
  \item{nstep}{(Not present for the \dQuote{bf} method.) The number
  of steps (iterations) actually used by the algorithm.  For the
  \dQuote{nl} method, it is the \code{iterations} component of the object
  returned by \code{\link{nlm}()}.}
  \item{counts}{Replaces \code{nstep} for the \dQuote{bf} method. It
  is the \code{counts} component of the object returned by
  \code{\link{optim}()}.}
  \item{mixture}{A logical scalar.  Was a mixture model fitted?}
  \item{expo}{A logical scalar.  Was the exponetial (logistic)
  parameterization used for the entries of the transition probability
  matrix?}
  \item{formula}{The formula for the model that was fitted; equal to
  the \code{formula} argument, augmented by \code{state}.}
  \item{contrasts}{The contrasts used in fitting the model.}
  \item{miss.frac}{The fraction or proportion of missing values in
  the observations.}
  \item{lambda}{A vector of estimated values of the Poisson parameter
  associated with each combination of observation and state.}
  \item{fy}{The Poisson probability of each observed value, for
  each state, calculated using the fitted parameters.}
  \item{data}{The data frame used to fit the model.  Constructed from
  the \code{data} argument supplied to this function by replicating
  the rows once for each state and adding a \code{state} column.
  A column \code{cf} (\dQuote{cell factor}) has also been added.
  This column indicates, by means of a single factor, which cell
  of the model a given row of \code{data} corresponds to.}
  \item{data.name}{The name of the data set to which the model
  was fitted.  Equal to the \code{data.name} argument or its
  default value.}
  \item{method}{The method (\dQuote{lm}, \dQuote{sw}, \dQuote{em},
  \dQuote{bf} or \dQuote{nl})
  used to fit the model.  Equal to the \code{method} argument or
  its default value.}
  \item{tolerance}{The convergence tolerance used to fit the model.
  Equal to the \code{tolerance} argument or its default value.}
  \item{cells}{A character vector indicating the names of the factors
  specifying the \dQuote{cells} of the model.  (Equal to the \code{cells}
  argument.)}
}
\references{
  T. Rolf Turner, Murray A. Cameron, and Peter J. Thomson.  Hidden
  Markov chains in generalized linear models.  Canadian J. Statist.,
  vol. 26, pp. 107 -- 125, 1998.

  Rolf Turner.  Direct maximization of the likelihood of a hidden
  Markov model.  Submitted for publication, 2007.
}

\author{Rolf Turner
  \email{r.turner@auckland.ac.nz}
}

\seealso{ \code{\link{fitted.hglmm}()} \code{\link{rhglmm.default}()}
          \code{\link{rhglmm.hglmm}()} \code{\link{bcov}()} }

\examples{
    loc4 <- c("LngRf","BondiE","BondiOff","MlbrOff")
    SCC4 <- SydColCount[SydColCount$locn \%in\% loc4,] 
    SCC4$locn <- factor(SCC4$locn) # Get rid of unused levels.
    rownames(SCC4) <- 1:nrow(SCC4)
    fit.lm <- hglmm(y~locn+depth,SCC4,c("locn","depth"),K=2,
                    verb=TRUE)
    fit.em <- hglmm(y~locn+depth,SCC4,c("locn","depth"),K=2,
                    method="em",verb=TRUE)
}
\keyword{ models }
